import { Subject } from 'rxjs';
import { DashboardViewBase } from '@/studio-core';
import HROperationUnitService from '@/service/hroperation-unit/hroperation-unit-service';
import HROperationUnitAuthService from '@/authservice/hroperation-unit/hroperation-unit-auth-service';
import PortalViewEngine from '@engine/view/portal-view-engine';
import HROperationUnitUIService from '@/uiservice/hroperation-unit/hroperation-unit-ui-service';

/**
 * 运营单位信息看板视图基类
 *
 * @export
 * @class HROperationUnitDashboardViewBase
 * @extends {DashboardViewBase}
 */
export class HROperationUnitDashboardViewBase extends DashboardViewBase {
    /**
     * 视图对应应用实体名称
     *
     * @protected
     * @type {string}
     * @memberof HROperationUnitDashboardViewBase
     */
    protected appDeName: string = 'hroperationunit';

    /**
     * 应用实体主键
     *
     * @protected
     * @type {string}
     * @memberof HROperationUnitDashboardViewBase
     */
    protected appDeKey: string = 'operationunitid';

    /**
     * 应用实体主信息
     *
     * @protected
     * @type {string}
     * @memberof HROperationUnitDashboardViewBase
     */
    protected appDeMajor: string = 'operationunitname';

    /**
     * 实体服务对象
     *
     * @type {HROperationUnitService}
     * @memberof HROperationUnitDashboardViewBase
     */
    protected appEntityService: HROperationUnitService = new HROperationUnitService;

    /**
     * 实体权限服务对象
     *
     * @type HROperationUnitUIService
     * @memberof HROperationUnitDashboardViewBase
     */
    public appUIService: HROperationUnitUIService = new HROperationUnitUIService(this.$store);


    /**
     * 计数器服务对象集合
     *
     * @protected
     * @type {Array<*>}
     * @memberof HROperationUnitDashboardViewBase
     */    
    protected counterServiceArray: Array<any> = [];

    /**
     * 视图模型数据
     *
     * @protected
     * @type {*}
     * @memberof HROperationUnitDashboardViewBase
     */
    protected model: any = {
        srfCaption: 'entities.hroperationunit.views.dashboardview.caption',
        srfTitle: 'entities.hroperationunit.views.dashboardview.title',
        srfSubTitle: 'entities.hroperationunit.views.dashboardview.subtitle',
        dataInfo: ''
    }

    /**
     * 容器模型
     *
     * @protected
     * @type {*}
     * @memberof HROperationUnitDashboardViewBase
     */
    protected containerModel: any = {
        view_dashboard: { name: 'dashboard', type: 'DASHBOARD' },
    };


	/**
     * 视图唯一标识
     *
     * @protected
     * @type {string}
     * @memberof ViewBase
     */
	protected viewtag: string = 'e249cd84a7d955affe21832bbe41c40a';


    /**
     * 视图引擎
     *
     * @public
     * @type {Engine}
     * @memberof HROperationUnitDashboardViewBase
     */
    public engine: PortalViewEngine = new PortalViewEngine();

    /**
     * 引擎初始化
     *
     * @public
     * @memberof HROperationUnitDashboardViewBase
     */
    public engineInit(): void {
        this.engine.init({
            view: this,
            dashboard: this.$refs.dashboard,
            keyPSDEField: 'hroperationunit',
            majorPSDEField: 'operationunitname',
            isLoadDefault: true,
        });
    }

    /**
     * dashboard 部件 load 事件
     *
     * @param {*} [args={}]
     * @param {*} $event
     * @memberof HROperationUnitDashboardViewBase
     */
    public dashboard_load($event: any, $event2?: any): void {
        this.engine.onCtrlEvent('dashboard', 'load', $event);
    }


}