import { Subject } from 'rxjs';
import { TabExpViewBase } from '@/studio-core';
import HRTransferApplyService from '@/service/hrtransfer-apply/hrtransfer-apply-service';
import HRTransferApplyAuthService from '@/authservice/hrtransfer-apply/hrtransfer-apply-auth-service';
import TabExpViewEngine from '@engine/view/tab-exp-view-engine';
import HRTransferApplyUIService from '@/uiservice/hrtransfer-apply/hrtransfer-apply-ui-service';

/**
 * 调动视图基类
 *
 * @export
 * @class HRTransferApplyTabExpViewBase
 * @extends {TabExpViewBase}
 */
export class HRTransferApplyTabExpViewBase extends TabExpViewBase {
    /**
     * 视图对应应用实体名称
     *
     * @protected
     * @type {string}
     * @memberof HRTransferApplyTabExpViewBase
     */
    protected appDeName: string = 'hrtransferapply';

    /**
     * 应用实体主键
     *
     * @protected
     * @type {string}
     * @memberof HRTransferApplyTabExpViewBase
     */
    protected appDeKey: string = 'hrtransferapplyid';

    /**
     * 应用实体主信息
     *
     * @protected
     * @type {string}
     * @memberof HRTransferApplyTabExpViewBase
     */
    protected appDeMajor: string = 'hrtransferapplyname';

    /**
     * 实体服务对象
     *
     * @type {HRTransferApplyService}
     * @memberof HRTransferApplyTabExpViewBase
     */
    protected appEntityService: HRTransferApplyService = new HRTransferApplyService;

    /**
     * 实体权限服务对象
     *
     * @type HRTransferApplyUIService
     * @memberof HRTransferApplyTabExpViewBase
     */
    public appUIService: HRTransferApplyUIService = new HRTransferApplyUIService(this.$store);


    /**
     * 计数器服务对象集合
     *
     * @protected
     * @type {Array<*>}
     * @memberof HRTransferApplyTabExpViewBase
     */    
    protected counterServiceArray: Array<any> = [];

    /**
     * 视图模型数据
     *
     * @protected
     * @type {*}
     * @memberof HRTransferApplyTabExpViewBase
     */
    protected model: any = {
        srfCaption: 'entities.hrtransferapply.views.tabexpview.caption',
        srfTitle: 'entities.hrtransferapply.views.tabexpview.title',
        srfSubTitle: 'entities.hrtransferapply.views.tabexpview.subtitle',
        dataInfo: ''
    }

    /**
     * 容器模型
     *
     * @protected
     * @type {*}
     * @memberof HRTransferApplyTabExpViewBase
     */
    protected containerModel: any = {
        view_tabexppanel: { name: 'tabexppanel', type: 'TABEXPPANEL' },
        wflinks: [],
    };


	/**
     * 视图唯一标识
     *
     * @protected
     * @type {string}
     * @memberof ViewBase
     */
	protected viewtag: string = '8e90bfab442fc1c89453722dfe094408';


    /**
     * 视图引擎
     *
     * @public
     * @type {Engine}
     * @memberof HRTransferApplyTabExpViewBase
     */
    public engine: TabExpViewEngine = new TabExpViewEngine();

    /**
     * 引擎初始化
     *
     * @public
     * @memberof HRTransferApplyTabExpViewBase
     */
    public engineInit(): void {
        this.engine.init({
            view: this,
            keyPSDEField: 'hrtransferapply',
            majorPSDEField: 'hrtransferapplyname',
            isLoadDefault: true,
        });
    }


}