package cn.ibizlab.humanresource.core.humanresource.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.humanresource.core.humanresource.domain.HREducation;
import cn.ibizlab.humanresource.core.humanresource.filter.HREducationSearchContext;
import cn.ibizlab.humanresource.core.humanresource.service.IHREducationService;

import cn.ibizlab.humanresource.util.helper.CachedBeanCopier;
import cn.ibizlab.humanresource.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.humanresource.core.humanresource.mapper.HREducationMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[教育信息] 服务对象接口实现
 */
@Slf4j
@Service("HREducationServiceImpl")
public class HREducationServiceImpl extends ServiceImpl<HREducationMapper, HREducation> implements IHREducationService {

    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.humanresource.service.IHREmployeeService hremployeeService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(HREducation et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getHreducationid()),et);
        return true;
    }

    @Override
    public void createBatch(List<HREducation> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(HREducation et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("hreducationid",et.getHreducationid())))
            return false;
        CachedBeanCopier.copy(get(et.getHreducationid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<HREducation> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public HREducation get(String key) {
        HREducation et = getById(key);
        if(et==null){
            et=new HREducation();
            et.setHreducationid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public HREducation getDraft(HREducation et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(HREducation et) {
        return (!ObjectUtils.isEmpty(et.getHreducationid()))&&(!Objects.isNull(this.getById(et.getHreducationid())));
    }
    @Override
    @Transactional
    public boolean save(HREducation et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(HREducation et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<HREducation> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<HREducation> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<HREducation> selectByHremployeeid(String employeeid) {
        return baseMapper.selectByHremployeeid(employeeid);
    }

    @Override
    public void removeByHremployeeid(String employeeid) {
        this.remove(new QueryWrapper<HREducation>().eq("hremployeeid",employeeid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<HREducation> searchDefault(HREducationSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<HREducation> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<HREducation>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(HREducation et){
        //实体关系[DER1N_HREDUCATION_HREMPLOYEE_HREMPLOYEEID_327A4D]
        if(!ObjectUtils.isEmpty(et.getHremployeeid())){
            cn.ibizlab.humanresource.core.humanresource.domain.HREmployee hremployee=et.getHremployee();
            if(ObjectUtils.isEmpty(hremployee)){
                cn.ibizlab.humanresource.core.humanresource.domain.HREmployee majorEntity=hremployeeService.get(et.getHremployeeid());
                et.setHremployee(majorEntity);
                hremployee=majorEntity;
            }
            et.setHremployeename(hremployee.getEmployeename());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<HREducation> getHreducationByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<HREducation> getHreducationByEntities(List<HREducation> entities) {
        List ids =new ArrayList();
        for(HREducation entity : entities){
            Serializable id=entity.getHreducationid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



